package ru.amse.gomoku.players.impl.aiPlayer;

import ru.amse.gomoku.board.IBoard;
import java.util.Random;

/**
 * the main part for searching one of the best moves.
 * using this class does not guarantee that move is perfect,
 * it is just not bad and sometimes even very good.
 */
class Looker {

    static final boolean ALLOW_RANDOMIZING = true;
    static final int MY_MAX = 1000000;
    private byte myColour;
    Estimator estimator;

    Looker(byte colour1) {

        estimator = new Estimator();
        myColour = colour1;
    }

    byte[] search(byte[][] board) {
        Random r = new Random();
        int size = board.length;
        byte heightNeeded = -1;
        byte widthNeeded = -1;

        int maxEstimated = -1;
        for (byte i = 0; i < size; i++ ) {
            for (byte j = 0; j < size; j++) {
                if (board[i][j] == IBoard.EMPTY_TURN) {
                    int estimated = estimator.estimate(i, j, myColour, board);
                    if ((estimated > maxEstimated)
                       || ((estimated == maxEstimated)
                          && (ALLOW_RANDOMIZING && r.nextBoolean()))) {
                        maxEstimated = estimated;
                        heightNeeded = i;
                        widthNeeded = j;
                    }
                }
            }
        }
        return new byte[]{heightNeeded, widthNeeded};
    }
}
