package ru.amse.gomoku.providers.impl;

import ru.amse.gomoku.providers.IImageProvider;

import javax.swing.*;
import java.net.URL;
import java.net.URLClassLoader;

public class ImageProvider implements IImageProvider {

    private ImageIcon[] myPlayerImages = new ImageIcon[NUM_PLAYER_IMAGES + 1];
    private ImageIcon[] myActionIcons = new ImageIcon[NUM_ACTION_ICONS];
    private ImageIcon myPersonImage;
    private int myDibNumber = 0;

    public ImageProvider() {
        loadPersonImage();
        loadDefaultPlayerImages();
        loadActionIcons();
    }

    private URL getURL(String fileName) {
		URLClassLoader urlLoader = (URLClassLoader) this.getClass()
				.getClassLoader();
        return urlLoader.findResource(fileName);
	}
    
    public void loadDefaultPlayerImages() {

        myPlayerImages[0] = myPersonImage;

        URL url = getURL("ru/amse/gomoku/images/notAperson.png");
        myPlayerImages[1] = new ImageIcon(url);

        url = getURL("ru/amse/gomoku/images/intellect.png");
        myPlayerImages[2] = new ImageIcon(url);

        url = getURL("ru/amse/gomoku/images/intellect1.png");
        myPlayerImages[3] = new ImageIcon(url);
    }

    public void loadPersonImage() {    	
        URL url = getURL("ru/amse/gomoku/images/person.png");
        myPersonImage = new ImageIcon(url, "Person");                
    }

    public void loadActionIcons() {
        URL url = getURL("ru/amse/gomoku/images/Run.png");
        myActionIcons[0] = new ImageIcon(url, "NewGame");

        url = getURL("ru/amse/gomoku/images/undo.png");
        myActionIcons[1] = new ImageIcon(url, "Undo");

        url = getURL("ru/amse/gomoku/images/play.png");
        myActionIcons[2] = new ImageIcon(url, "Play");

        url = getURL("ru/amse/gomoku/images/pause.png");
        myActionIcons[3] = new ImageIcon(url, "Pause");

        url = getURL("ru/amse/gomoku/images/draw.jpg");
        myActionIcons[4] = new ImageIcon(url, "Draw");

        url = getURL("ru/amse/gomoku/images/ok.png");
        myActionIcons[5] = new ImageIcon(url, "Win");

        url = getURL("ru/amse/gomoku/images/add.png");
        myActionIcons[6] = new ImageIcon(url, "Add");
        
        url = getURL("ru/amse/gomoku/images/remove.png");
        myActionIcons[7] = new ImageIcon(url, "Remove");    

        String path = "ru/amse/gomoku/images/dib";
        String name = "Dib";
        for (int i = 1; i < 8; i++) {
            url = getURL(path + i + ".png");
            myActionIcons[i + 7] = new ImageIcon(url, name + i);
            myDibNumber++;
        }
        url = getURL("ru/amse/gomoku/images/stop.png");
        myActionIcons[15] = new ImageIcon(url, "Stop");
        
        url = getURL("ru/amse/gomoku/images/ok.png");
        myActionIcons[16] = new ImageIcon(url, "Ok");
    }

    public ImageIcon[] getPlayerImages() {
        return myPlayerImages;
    }

    public ImageIcon getActionIcon(String name) {
        for (ImageIcon actionIcon: myActionIcons) {
            if (actionIcon.getDescription().equals(name)) {
                return actionIcon;
            }
        }
        return null;
    }

    public int getDibNumber() {
        return myDibNumber;
    }

    public ImageIcon getPersonImage() {
        return myPersonImage;
    }
}
