package ru.amse.gomoku.providers.impl;

import ru.amse.gomoku.board.IBoard;
import ru.amse.gomoku.players.IPlayer;
import ru.amse.gomoku.players.impl.AIPlayer;
import ru.amse.gomoku.providers.IIntellectProvider;
import ru.amse.gomoku.*;

import javax.swing.*;
import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.LinkedList;

/**
 *
 */
public class IntellectProvider extends URLClassLoader implements IIntellectProvider {

    private Hashtable<String, IPlayer> myPlayers = new Hashtable<String, IPlayer>();
    private LinkedList<File> myAddedPlayers = new LinkedList<File>();
    private int mySize = 0;
    private static Main m = new Main();

    public IntellectProvider(){
        super(new URL[0], m.getClass().getClassLoader());
    }

    public void registerPlayer(String name, IPlayer player) throws IllegalAccessException {
        if (myPlayers.containsKey(name)) {
            throw new IllegalAccessException("Trying to add a player with existing name!");
        }
        myPlayers.put(name, player);
        mySize++;
    }

    public void registerPlayer(File playerFile) {
        try {
            this.addURL(new URL("file:" + playerFile.getPath()));
            Class c = this.loadClass(playerFile.getName()
                    .substring(0, playerFile.getName().lastIndexOf(".jar")));
            IPlayer aip = (IPlayer) c.newInstance();
            aip.setName(playerFile.getName());
            registerPlayer(aip.getName(), aip);
            myAddedPlayers.add(playerFile);
        }
        catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
        catch (MalformedURLException e) {}
        catch (InstantiationException e) {}
        catch (IllegalAccessException e) {}
    }

    public void unRegisterPlayer(String name) {
        if (myPlayers.remove(name) != null) {
            mySize--;
        }
    }

    public IPlayer getPlayer(String name) {
        IPlayer toGet;
        try {
            toGet = myPlayers.get(name).getClass().newInstance();
        } catch (Exception ee) {
            return null;
        }
        toGet.setName(name);
        return toGet;
    }

    public ImageIcon getPlayerImage(String playerName) {
        IPlayer player = myPlayers.get(playerName);
        return player.getImage();
    }

    public String[] getAllNames() {        
        Enumeration<String>  enu = myPlayers.keys();
        String[] names = new String[myPlayers.size()];

        for (int i = 0; i < names.length; i++) {
            names[i] = enu.nextElement();
        }
        return names;
    }

    public LinkedList<File> getAddedPlayers() {
        return myAddedPlayers;
    }

    public int size() {
        return mySize;
    }
}
